#****************************************************************************
#**
#**  File     :  /cdimage/units/UEL0201/UEL0201_script.lua
#**  Author(s):  John Comes, David Tomandl, Jessica St. Croix
#**
#**  Summary  :  UEF Medium Tank Script
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local TLandUnit = import('/lua/terranunits.lua').TLandUnit
local DefaultUnitsFile = import('/lua/defaultunits.lua')
local TDFGaussCannonWeapon = import('/lua/terranweapons.lua').TDFGaussCannonWeapon
local TDFMachineGunWeapon = import('/lua/terranweapons.lua').TDFMachineGunWeapon
local Buff = import('/lua/sim/Buff.lua')

local hasHEBI = false # Flag signalant Enhancement avec Heavy Electron Bolter
local hasHEBII = false
local hasHEBI = false
local hasHEBIV = false
local hasHEBV = false

UEL0108 = Class(TLandUnit) {

    Weapons = {
        MainGun = Class(TDFGaussCannonWeapon) {
			SetOnTransport = function(self, transportstate)
				self.unit:SetScriptBit('RULEUTC_WeaponToggle', false)    
			end,  
		},
		ArmCannonTurret = Class(TDFMachineGunWeapon) {},
	},
	
    OnStopBeingBuilt = function(self,builder,layer)
		TLandUnit.OnStopBeingBuilt(self,builder,layer)		
		self:SetWeaponEnabledByLabel('ArmCannonTurret', true)
		self:SetWeaponEnabledByLabel('MainGun', false)
		IssueClearCommands({self})
		self:AddCommandCap('RULEUCC_Move')
		self:SetSpeedMult(1.0)
		self:SetTurnMult(1.0)
		self:AddUnitCallback(self.OnVeteran, 'OnVeteran') # Ajouter 1 Trigger pour lancer Enhancement
	end,
	
    OnCreate = function(self)
        local bp = self:GetBlueprint()	
        TLandUnit.OnCreate(self)		
        if not self.AnimationManipulator then
            self.AnimationManipulator = CreateAnimator(self)
            self.Trash:Add(self.AnimationManipulator)
        end
        self.AnimationManipulator:PlayAnim(bp.Display.AnimationActivate, false):SetRate(0)	
        self:SetWeaponEnabledByLabel('MainGun', false)
		IssueClearCommands({self})
		self:AddCommandCap('RULEUCC_Move')
		self:SetSpeedMult(1.0)
		self:SetTurnMult(1.0)
			self.hasHEBI = false
			self.hasHEBII = false
			self.hasHEBIII = false
			self.hasHEBIV = false
			self.hasHEBV = false
    end,
	
        
    OnScriptBitSet = function(self, bit)
        TLandUnit.OnScriptBitSet(self, bit)
        if bit == 1 then
    	    if self.AnimationManipulator then			
                self:SetUnSelectable(true)				
				IssueClearCommands({self})
				self:RemoveCommandCap('RULEUCC_Move')
				self:SetSpeedMult(0)
				self:SetTurnMult(0)
                self.AnimationManipulator:SetRate(1.5)   
                self:ForkThread(function()
                    WaitSeconds(self.AnimationManipulator:GetAnimationDuration()*self.AnimationManipulator:GetRate())              
                    self:SetUnSelectable(false)
                end)
				self:SetWeaponEnabledByLabel('MainGun', true)
				
            end 		
    	end
    end,
    OnScriptBitClear = function(self, bit)
        TLandUnit.OnScriptBitClear(self, bit)
        if bit == 1 then 
    	    if self.AnimationManipulator then	
                self:SetUnSelectable(true)				
                IssueClearCommands({self})
                self:AddCommandCap('RULEUCC_Move')				
		    	self:SetSpeedMult(1.0)
                self:SetTurnMult(1.0) 		
                self.AnimationManipulator:SetRate(-1.5)
                self:ForkThread(function()
                    WaitSeconds(self.AnimationManipulator:GetAnimationDuration()*self.AnimationManipulator:GetRate())
                    self:SetUnSelectable(false)
                end)		
				self:SetWeaponEnabledByLabel('MainGun', false)
            end  
    	end
    end,

	#Level#
	OnVeteran = function ( self )
        local bp = self:GetBlueprint()
		local enh = 'VeterancyI'
		local bpEnh = bp.ExpeWars_Enhancement[enh]
		if not bpEnh then return end
        local bpEnhEAVLevel = bpEnh.EnabledAtVeterancyLevel
		if bpEnhEAVLevel and bpEnhEAVLevel > 0 and ( self.VeteranLevel == bpEnhEAVLevel ) then
			if enh =='VeterancyI' then
				hasHEBI = true    # Signale HEB INSTALL
				hasHEBII = false
				hasHEBIII = false
				hasHEBIV = false
				hasHEBV = false
                BuffBlueprint {
                    Name = 'UEFHEALTHBUFF',
                    DisplayName = 'UEFHEALTHBUFF',
                    BuffType = 'MAXHEALTH',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1,
                        },
                    },
                }
            end
			Buff.ApplyBuff(self, 'UEFHEALTHBUFF')
		end
		######################
        local bp = self:GetBlueprint()
		local enh = 'VeterancyII'
		local bpEnh = bp.ExpeWars_Enhancement[enh]
		if not bpEnh then return end
        local bpEnhEAVLevel = bpEnh.EnabledAtVeterancyLevel
		# ADDS MY CUSTOM ENHANCEMENT !
		if bpEnhEAVLevel and bpEnhEAVLevel > 0 and ( self.VeteranLevel == bpEnhEAVLevel ) then
			if enh =='VeterancyII' then
				local wep = self:GetWeaponByLabel('MainGun')
				local wep2 = self:GetWeaponByLabel('ArmCannonTurret')
				wep:ChangeRateOfFire(bp.NewRateOfFireI or 1.0)
				wep2:ChangeRateOfFire(bp.NewRateOfFireII or 4.0)
				hasHEBI = true
				hasHEBII = true    # Signale HEB INSTALL
				hasHEBIII = false
				hasHEBIV = false
				hasHEBV = false
			end
		end
		######################
        local bp = self:GetBlueprint()
		local enh = 'VeterancyIII'
		local bpEnh = bp.ExpeWars_Enhancement[enh]
		if not bpEnh then return end
        local bpEnhEAVLevel = bpEnh.EnabledAtVeterancyLevel
		# ADDS MY CUSTOM ENHANCEMENT !
		if bpEnhEAVLevel and bpEnhEAVLevel > 0 and ( self.VeteranLevel == bpEnhEAVLevel ) then
			if enh =='VeterancyIII' then
				local wep = self:GetWeaponByLabel('MainGun')
				local wep2 = self:GetWeaponByLabel('ArmCannonTurret')
    		    wep:AddDamageMod(bpEnh.BolterDamageModI)       
				wep2:AddDamageMod(bpEnh.BolterDamageModII)       
				hasHEBI = true
				hasHEBII = true    # Signale HEB INSTALL
				hasHEBIII = true
				hasHEBIV = false
				hasHEBV = false
			end
		end
		#######################
        local bp = self:GetBlueprint()
		local enh = 'VeterancyIV'
		local bpEnh = bp.ExpeWars_Enhancement[enh]
		if not bpEnh then return end
        local bpEnhEAVLevel = bpEnh.EnabledAtVeterancyLevel
		# ADDS MY CUSTOM ENHANCEMENT !
		if bpEnhEAVLevel and bpEnhEAVLevel > 0 and ( self.VeteranLevel == bpEnhEAVLevel ) then
			if enh =='VeterancyIV' then
				hasHEBI = true
				hasHEBII = true    # Signale HEB INSTALL
				hasHEBIII = true
				hasHEBIV = true
				hasHEBV = false
                BuffBlueprint {
                    Name = 'UEFHEALTHBUFF',
                    DisplayName = 'UEFHEALTHBUFF',
                    BuffType = 'MAXHEALTH',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.2,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1,
                        },
                    },
                }
            end
			Buff.ApplyBuff(self, 'UEFHEALTHBUFF')
		end
		#######################
        local bp = self:GetBlueprint()
		local enh = 'VeterancyV'
		local bpEnh = bp.ExpeWars_Enhancement[enh]
		if not bpEnh then return end
        local bpEnhEAVLevel = bpEnh.EnabledAtVeterancyLevel
		# ADDS MY CUSTOM ENHANCEMENT !
		if bpEnhEAVLevel and bpEnhEAVLevel > 0 and ( self.VeteranLevel == bpEnhEAVLevel ) then
			if enh =='VeterancyV' then
				local wep = self:GetWeaponByLabel('MainGun')
				local wep2 = self:GetWeaponByLabel('ArmCannonTurret')
    		    wep:AddDamageMod(bpEnh.BolterDamageModI)       
				wep2:AddDamageMod(bpEnh.BolterDamageModII)  
				wep:ChangeRateOfFire(bp.NewRateOfFireI or 1.3)
				wep2:ChangeRateOfFire(bp.NewRateOfFireII or 5.0)
				hasHEBI = true
				hasHEBII = true    # Signale HEB INSTALL
				hasHEBIII = true
				hasHEBIV = true
				hasHEBV = true
			end
		end
	end,	
	
}

TypeClass = UEL0108
